<?php

/**
 * This file contains the main include file for the Inputbox extension of 
 * MediaWiki. 
 *
 * Usage: require_once("path/to/inputbox.php"); in LocalSettings.php
 *
 * This extension requires MediaWiki 1.5 or higher.
 *
 * @author Erik Moeller <moeller@scireview.de>
 *  namespaces search improvements partially by
 *  Leonardo Pimenta <leo.lns@gmail.com> 
 * @copyright Public domain
 * @license Public domain
 * @version 0.1.1
 * @version 0.1.2 patch for MW 1.12 and later
 * @version 0.1.3 changed URL of homepage to Extension:InputBox
 */

// define as a Parser FUNCTION
$wgExtensionFunctions[]        		= 'InputBoxMaster::register_PF';
$wgHooks['LanguageGetMagic'][] 		= 'InputBoxMaster::register_PF_Magic';
$wgHooks['OutputPageBeforeHTML'][]	= 'InputBoxMaster::postProcess';

// define as a Parser EXTENSION (user tag)
$wgExtensionFunctions[] 			= 'InputBoxMaster::register_PE';
$wgExtensionCredits['parserhook'][] = array(
	'name' => 'Inputbox',
	'author' => 'Erik Moeller',
	'url' => 'http://meta.wikimedia.org/wiki/Extension:InputBox',
	'description' => 'Allow inclusion of predefined HTML forms.',
);


class InputBoxMaster {

	
	
	// register as a Parser FUNCTION
	static function register_PF() {
		global $wgParser;
		$wgParser->setFunctionHook( 'inputbox', 'InputBoxMaster::render_PF' );
		global $myWgHtmlEscFlag;
		if ($myWgHtmlEscFlag==0) {
			$myWgHtmlEscFlag = rand(10000000,99999999); // 9531045;
		}
	}

	 
	# Search and replace the XML comments with the hidden, encoded content
	static function postProcess( &$out, &$text ) {
		global $wgParser;
		global $myWgHtmlEscFlag;
		$text = preg_replace('/&lt;\/?root&gt;/','',str_replace('~2'.$myWgHtmlEscFlag,'>',str_replace('~1'.$myWgHtmlEscFlag,'<',$text)));
		return true;
	}


	// register alias names for Parser FUNCTION
	static function register_PF_Magic( &$magicWords, $langCode ) {
			# The first array element is case sensitivity, in this case it is not case sensitive
			# All remaining elements are synonyms for our parser function
			$magicWords['inputbox'] = array( 0, 'inputbox' );
			return true;
	}

	// entry point for Parser FUNCTION
	static function render_PF(&$parser, $p1='' ,$p2='' ,$p3='' ,$p4='' ,$p5='' ,$p6='' ,$p7='' ,$p8='' ,$p9='' ,$p10='',
										  $p11='',$p12='',$p13='',$p14='',$p15='',$p16='',$p17='',$p18='',$p19='',$p20='') {
  		global $myWgHtmlEscFlag;

		$params = array();
		global $wgVersion;
		
		$input='';
		if ($p1 != '')	$input .= str_replace("\n",'',$p1) ."\n";
		if ($p2 != '')	$input .= str_replace("\n",'',$p2) ."\n";
		if ($p3 != '')	$input .= str_replace("\n",'',$p3) ."\n";
		if ($p4 != '')	$input .= str_replace("\n",'',$p4) ."\n";
		if ($p5 != '')	$input .= str_replace("\n",'',$p5) ."\n";
		if ($p6 != '')	$input .= str_replace("\n",'',$p6) ."\n";
		if ($p7 != '')	$input .= str_replace("\n",'',$p7) ."\n";
		if ($p8 != '')	$input .= str_replace("\n",'',$p8) ."\n";
		if ($p9 != '')	$input .= str_replace("\n",'',$p9) ."\n";
		if ($p10!= '')	$input .= str_replace("\n",'',$p10)."\n";
		if ($p11!= '')	$input .= str_replace("\n",'',$p11)."\n";
		if ($p12!= '')	$input .= str_replace("\n",'',$p12)."\n";
		if ($p13!= '')	$input .= str_replace("\n",'',$p13)."\n";
		if ($p14!= '')	$input .= str_replace("\n",'',$p14)."\n";
		if ($p15!= '')	$input .= str_replace("\n",'',$p15)."\n";
		if ($p16!= '')	$input .= str_replace("\n",'',$p16)."\n";
		if ($p17!= '')	$input .= str_replace("\n",'',$p17)."\n";
		if ($p18!= '')	$input .= str_replace("\n",'',$p18)."\n";
		if ($p19!= '')	$input .= str_replace("\n",'',$p19)."\n";
		if ($p20!= '')	$input .= str_replace("\n",'',$p20)."\n";

		#encrypt <..>
		$result = str_replace('>','~2'.$myWgHtmlEscFlag,str_replace('<','~1'.$myWgHtmlEscFlag,InputBoxMaster::render( $input, $params, $parser )));
		return $parser->getPreprocessor()->preprocessToObj($result, Parser::PTD_FOR_INCLUSION );
	}


	// register as a Parser EXTENSION
	static function register_PE() {
		global $wgParser;
		$wgParser->setHook('inputbox', 'InputBoxMaster::render');
	}
	  
	/**
	 * Renders an inputbox based on information provided by $input.
	 */
	static function render($input, $params, &$parser)
	{
		global $wgContLang;
		$parser->disableCache();
		
		$inputbox=new Inputbox( $parser );
		self::getBoxOption($inputbox->type,$input,'type',$parser);
		self::getBoxOption($inputbox->pageFrom,$input,'from',$parser);
		self::getBoxOption($inputbox->pageTo,$input,'to',$parser);
		self::getBoxOption($inputbox->changeLinks,$input,'changeLinks',$parser);
		self::getBoxOption($inputbox->forced,$input,'forced',$parser);
		self::getBoxOption($inputbox->pattern,$input,'pattern',$parser);
		self::getBoxOption($inputbox->replacement,$input,'replacement',$parser);
		self::getBoxOption($inputbox->width,$input,'width',$parser,true);	
		self::getBoxOption($inputbox->preload,$input,'preload',$parser);
		self::getBoxOption($inputbox->editintro,$input,'editintro',$parser);
		self::getBoxOption($inputbox->defaulttext,$input,'default',$parser);	
		self::getBoxOption($inputbox->titleprefix,$input,'titleprefix',$parser);
		self::getBoxOption($inputbox->bgcolor,$input,'bgcolor',$parser);
		self::getBoxOption($inputbox->buttonlabel,$input,'buttonlabel',$parser);	
		self::getBoxOption($inputbox->searchbuttonlabel,$input,'searchbuttonlabel',$parser);		
		self::getBoxOption($inputbox->namespaces,$input,'namespaces',$parser);		
		self::getBoxOption($inputbox->id,$input,'id',$parser);	
		self::getBoxOption($inputbox->labeltext,$input,'labeltext',$parser);
		self::getBoxOption($inputbox->br, $input, 'break',$parser);
		self::getBoxOption($inputbox->hidden, $input, 'hidden',$parser);
		self::getBoxOption($inputbox->target, $input, 'target',$parser);
		$inputbox->lineBreak();
		$inputbox->checkWidth();
		
		$boxhtml=$inputbox->render();
		# Maybe support other useful magic words here
		$boxhtml=str_replace("{{PAGENAME}}",$parser->getTitle()->getText(),$boxhtml);
		$namespaces=$wgContLang->getNamespaces();
		$boxhtml=str_replace("{{NAMESPACE}}",$namespaces[$parser->getTitle()->getNamespace()],$boxhtml);
		if($boxhtml) {
			return $boxhtml;
		} else {
			return '<div><strong class="error">Input box: type not defined.</strong>('.$input.')</div>';
		}
	}

	static function getBoxOption(&$value, &$input, $name, &$parser, $isNumber=false) {

		  if(preg_match("/^\s*$name\s*=\s*(.*)/mi",$input,$matches)) {
			if($isNumber) {
				$value=intval($matches[1]);
			} else {
				$value=htmlspecialchars($matches[1]);
			}
			$value = str_replace("{{PAGENAME}}",$parser->getTitle()->getText(),$value);
		}
	}
}

class Inputbox {
	var $type,$width,$preload,$editintro, $br;
	var $defaulttext,$titleprefix,$bgcolor,$buttonlabel,$searchbuttonlabel;
	var $hidden;
	var $pageFrom,$pageTo, $changeLinks, $forced, $pattern, $replacement;
	
	static $formId = 0;
	
	function Inputbox( &$parser ) {
		$this->parser =& $parser;
	}
	
	function render() {
		if($this->type=='create' || $this->type=='comment') {
			return $this->getCreateForm();		
		} elseif($this->type=='search') {
			return $this->getSearchForm();
		} elseif($this->type=='search2') {
			return $this->getSearchForm2();
		} elseif($this->type=='redirect') {
			return $this->createRedirectPage($this->pageFrom,$this->pageTo,$this->forced);
		} elseif($this->type=='copy') {
			return $this->copyPage($this->pageFrom,$this->pageTo,$this->changeLinks,$this->forced);
		} elseif($this->type=='update') {
			return $this->updatePage($this->pageFrom,$this->pattern,$this->replacement);
		} else {
			return false;
		}	
	}
	
	function getSearchForm() {
		global $wgUser, $wgContLang;
		
		$sk=$wgUser->getSkin();
		$searchpath = $sk->escapeSearchLink();		
		if(!$this->buttonlabel) {
			$this->buttonlabel = str_replace("&amp;lt;","<",str_replace("&amp;gt;",">",wfMsgHtml( "tryexact" )));
		}
		if(!$this->searchbuttonlabel) {
			$this->searchbuttonlabel = str_replace("&amp;lt;","<",str_replace("&amp;gt;",">",wfMsgHtml( "searchfulltext" )));
		}


		$type = $this->hidden ? 'hidden' : 'text';
		$searchform=<<<ENDFORM
		<table border="0" width="100%" cellspacing="0" cellpadding="0">
		<tr>
		<td bgcolor="{$this->bgcolor}">
		<form name="searchbox" action="$searchpath" class="searchbox">
		<input class="searchboxInput" name="search" type="{$type}"
		value="{$this->defaulttext}" size="{$this->width}" />{$this->br}
ENDFORM;

		// disabled when namespace filter active
		$gobutton=<<<ENDGO
<input type='submit' name="go" class="searchboxGoButton" value="{$this->buttonlabel}" />&nbsp;
ENDGO;
		// Determine namespace checkboxes
		$namespaces = $wgContLang->getNamespaces();
		$namespacesarray = explode(",",$this->namespaces);

		// Test if namespaces requested by user really exist
		$searchform2 = '';
		if ($this->namespaces) {
			foreach ($namespacesarray as $usernamespace) {
				$checked = '';
				// Namespace needs to be checked if flagged with "**" or if it's the only one
				if (strstr($usernamespace,'**') || count($namespacesarray)==1) {
										$usernamespace = str_replace("**","",$usernamespace);
										$checked =" checked";
								}
				foreach ( $namespaces as $i => $name ) {
					if ($i < 0){
						continue;
					}elseif($i==0) {
						$name='Main';
					}
					if ($usernamespace == $name) {
						$searchform2 .= "<input type=\"checkbox\" name=\"ns{$i}\" value=\"1\"{$checked}>{$usernamespace}";
					}
				}
			}
			//Line feed 
			$searchform2 .= $this->br;		
			//If namespaces are defined remove the go button 
			//because go button doesn't accept namespaces parameters 
			$gobutton='';
		} 
		$searchform3=<<<ENDFORM2
		{$gobutton}
		<input type='submit' name="fulltext" class="searchboxSearchButton" value="{$this->searchbuttonlabel}" />
		</form>
		</td>
		</tr>
		</table>
ENDFORM2;
		//Return form values
		return $searchform . $searchform2 . $searchform3;
	}

	function getSearchForm2() {
		global $wgUser;
		
		$sk=$wgUser->getSkin();
		$searchpath = $sk->escapeSearchLink();		
		if(!$this->buttonlabel) {
			$this->buttonlabel = str_replace("&amp;lt;","<",str_replace("&amp;gt;",">",wfMsgHtml( "tryexact" )));
		}

		$output = $this->parser->parse( $this->labeltext,
			$this->parser->getTitle(), $this->parser->getOptions(), false, false );
		$this->labeltext = $output->getText();
		$this->labeltext = str_replace('<p>', '', $this->labeltext);
		$this->labeltext = str_replace('</p>', '', $this->labeltext);
		
		$type = $this->hidden ? 'hidden' : 'text';
		$searchform=<<<ENDFORM
<form action="$searchpath" class="bodySearch" id="bodySearch{$this->id}"><div class="bodySearchWrap"><label for="bodySearchIput{$this->id}">{$this->labeltext}</label><input type="{$type}" name="search" size="{$this->width}" class="bodySearchIput" id="bodySearchIput{$this->id}" /><input type="submit" name="go" value="{$this->buttonlabel}" class="bodySearchBtnGo" />
ENDFORM;

		if ( !empty( $this->fulltextbtn ) ) // this is wrong...
			$searchform .= '<input type="submit" name="fulltext" class="bodySearchBtnSearch" value="{$this->searchbuttonlabel}" />';

		$searchform .= '</div></form>';

		return $searchform;
	}

	
	function getCreateForm() {
		global $wgScript;	
		
		$action = htmlspecialchars( $wgScript );		
		if($this->type=="comment") {
			$comment='<input type="hidden" name="section" value="new" />';
			if(!$this->buttonlabel) {
				$this->buttonlabel = str_replace("&amp;lt;","<",str_replace("&amp;gt;",">",wfMsgHtml( "postcomment" )));
			}
		} else if ($this->type=="redirect") {
			$comment='';
			if(!$this->buttonlabel) {			
				$this->buttonlabel = str_replace("&amp;lt;","<",str_replace("&amp;gt;",">",wfMsgHtml( "createredirect" )));;
			}
		} else {
			$comment='';
			if(!$this->buttonlabel) {			
				$this->buttonlabel = str_replace("&amp;lt;","<",str_replace("&amp;gt;",">",wfMsgHtml( "createarticle" )));
			}
		}		
		$type = $this->hidden ? 'hidden' : 'text';
		$target = ($this->target == '') ? '' : ' target="'.$this->target.'"';
		$id = Inputbox::$formId;
		$id++;
		Inputbox::$formId = $id;		
		$createform=<<<ENDFORM
<table border="0" width="100%" cellspacing="0" cellpadding="0">
<tr>
<td bgcolor="{$this->bgcolor}">
<form name="createbox{$id}" action="$action" $target method="get" class="createbox" 
	onsubmit="document.createbox{$id}.title.value = document.createbox{$id}.titleprefix.value + document.createbox{$id}.title.value;">
	<input type='hidden' name="action" value="edit" />
	<input type="hidden" name="preload" value="{$this->preload}" />
	<input type="hidden" name="editintro" value="{$this->editintro}" />
	<input type='hidden' name="titleprefix" value="{$this->titleprefix}" />
	{$comment}
	<input class="createboxInput" name="title" type="{$type}"
	value="{$this->defaulttext}" size="{$this->width}" />{$this->br}
	<input type='submit' name="create" class="createboxButton"
	value="{$this->buttonlabel}" />
</form>
</td>
</tr>
</table>
ENDFORM;
		return $createform;
	}

	function lineBreak() {
		# Should we be inserting a <br /> tag?
		$cond = ( strtolower( $this->br ) == "no" );
		$this->br = $cond ? '' : '<br />';
	}

	function copyPage($from, $to, $changeLinks, $forced) {
		if ($from=="" || $to=="") return ("Cannot create page: 'from' and/or 'to' not specified");
		global $wgArticle;

		$fromTitle = Title::newFromText($from);
		if (!($fromTitle->exists())) {
			return ("Cannot copy page <big>$from</big> --- page not found.");
		}
		$fromArticle = new Article($fromTitle);

		$rawText = $fromArticle->getRawText();
		
		// change the links in the text if desired
		if (isset($this->changeLinks) && $this->changeLinks!="" && $this->changeLinks!=null) {
			$rawText= preg_replace("/\\[\\[([-'\",.A-Za-z _0-9]+)\\|/","[[".$this->changeLinks."\\1|",$rawText);
			$rawText= preg_replace("/\\[\\[([-'\",.A-Za-z _0-9]+)\\]\\]/","[[".$this->changeLinks."\\1|\\1]]",$rawText);
		}
		
		$toTitle   = Title::newFromText($to);
		
		if (!($toTitle->exists()) || $forced=="yes") {
			// only save if article does not yet exist
			if ($this->updateLogin('Redirector','t7oU8.i')) {
				$article = new Article($toTitle);
				$rc =$article->doEdit($rawText, 'HelperBot');
				return ("page copied from <big>[[$from]]</big> to <big>[[$to]]</big>.");
			} else {
				return ("Cannot create page; authentication failed.");
			}
		}
		else {
			return ("Cannot create page; <big>[[$to]]</big> already exists (forced=$forced).");
		}
		return "never get here";
	}

	function updatePage($from, $pattern, $replacement) {
		if ($from=="" || $pattern=="") return ("page name or pattern missing");
		global $wgArticle;

		$fromTitle = Title::newFromText($from);
		if (!($fromTitle->exists())) {
			return ("Cannot update page <big>$from</big> --- page not found.");
		}
		$fromArticle = new Article($fromTitle);

		$rawText = $fromArticle->getRawText();
		
		// change the text
		$replacement = str_replace("\\n","\n",$replacement);
		$replacement = str_replace("_"," ",$replacement);
		$rawText= preg_replace($pattern,$replacement,$rawText);
		
		$rc =$fromArticle->doEdit($rawText, 'HelperBot');
		return ("updated page <big>[[$from]]</big>.");
	}

	function createRedirectPage($page, $to, $forced) {
		if ($page=="" || $to=="") return ("Cannot create redirection: 'from' and/or 'to' not specified");
		global $wgArticle;
		$rawText = "#redirect [[$to]]\n";
		$title   = Title::newFromText($page);
		
		if (!($title->exists()) || $forced=="yes") {
			// only save if article does not yet exist
			if ($this->updateLogin('Redirector','t7oU8.i')) {
				$article = new Article($title);
				$rc =$article->doEdit($rawText, 'HelperBot');
				return ("redirection created from <big>[[$title]]</big> to <big>[[$to]]</big>.");
			} else {
				return ("Cannot create redirection; authentication failed.");
			}
		}
		else {
			return ("Cannot create redirection; page <big>[[$title]]</big> already exists (forced=$forced).");
		}
		return "never get here";
	}

	function updateLogin($user, $password) {
		$u = User::newFromName( $user );
		if( is_null( $u ) ) return false;
		if (!$u->checkPassword( $password )) return false;
		if (!in_array('user', $u->getEffectiveGroups())) return false;
		global $wgUser;
		$wgUser = $u;
		return true;
	}


	/**
	 * If the width is not supplied, set it to 50
	 */
	function checkWidth() {
		if( !$this->width || trim( $this->width ) == '' )
			$this->width = 50;
	}	
}