<?php
/**
 * this class contains the parser for Parsing...
 * - the Request's wpTextbox1's value when returning from the standard editor
 * - the editpage when edit session is started.
 * - the values of the currently inspected template
 *
 * - NOT the Template Definition Page which is already inspected by
 *       the class clsTypeDefinition
 */
class TemplateFormParser{

  /* INTERFACE:
   * static :   splitByTemplate ( $ text )
   * static :   getValuesFromText ( & $ templates , $ Segments )
   *
   */

  /**
   * finds all template-usages in $text the result is an array
   * of segments of $text 
   * assured is:  $text == implode("",splitByTemplate($text));
   *
   * @param $text - any wikitext
   *
   * @return an array containing the same text as the input parameter
   *       - broken into several sections
   */
  public static function splitByTemplate($text){
    $result = null;
    
    $index = self::findnexttemplate($text, 0);
    
    // text portion before the first occurence of the template
    $result[] = substr($text,0,$index);
    $text=substr($text,$index);
    
    while( (strlen($text)>0) && (strpos($text,"{{") == 0) ){
	  
      $i = 2;
      $brackets =2; // level = 2 as we have two opening brackets for the template call
      $max =strlen($text);
      
      for(;($i < $max) && ($brackets > 0);$i++){
		$char = substr($text,$i,1);
		if ($char=="{") $brackets++;
		if ($char=="}") $brackets--;
      }
      
	  $template =substr($text,0,$i);
	  if(clsTemplateForm::isFormTemplate($template)){
		$result[]=substr($text,0,$i);
		$text = substr($text, $i); // remaining text
		$i=0;
      }
	  
      // look for next occurence of template
      $index = self::findnexttemplate($text, $i);
      
	  if ( $i > 0)
		$result[count($result)-1] .= substr($text,0,$index);
	  else {
		if ( $index > 0)	$result[] = substr($text,0,$index); // append final text portion to the result array
		else				$result[] = '';
      }
      $text=substr($text,$index);
    }

	//Debug
	// foreach ( $result as $item) clsTemplateForm::debug("segment= ".$item);
	
	return $result;
  }



  /**
   * finds the first beginning of a template which is not enclosed by <nowiki>
   *
   * @param text
   * @param offset
   *
   * returns the position of the next Template inside $text beginning 
   *         the search after $offset
   */
  private static function findnexttemplate(&$text,$offset){
	
    do{
      $repeat = false;
      $index = strpos($text,"{{",$offset);
      if ( ($index) || (substr($text, $offset, 2) == "{{") ) {
		
		$tag = strpos($text,"<nowiki>",$offset);
		if ( ($tag) || (substr($text,$offset,8) == "<nowiki>" )){
		  if ($tag < $index){      
			// möglicherweise ist das nächste Template 
			//in <nowiki> eingeschlossen.
			if( strpos($text,"</nowiki>",$tag) ){
			  $depth=0;
			  $tag=$tag+2;
			  
			  do{
				if ((strpos($text,"<nowiki>",$tag))
					&& ( strpos($text,"<nowiki>",$tag)
						 < strpos($text,"</nowiki>",$tag))){
				  $tag = strpos($text,"<nowiki>",$tag)+2;
				  $depth++;
				}
				if (strpos($text,"</nowiki>",$tag))
				  if (( strpos($text,"</nowiki>",$tag)
						< strpos($text,"<nowiki>",$tag))
					  ||(!strpos($text,"<nowiki>",$tag))){
					$tag = strpos($text,"</nowiki>",$tag);
					$depth--;
				  }
			  }while($depth > 0);
			  
			  //	      if ($depth==0){
			  $tag = strpos($text,"</nowiki>",$tag);
			  //	      }
			  $offset=$tag;
			  $repeat = true;
			}
			else 
			  return $index;
		  }
		}
      } else {
		$index=strlen($text);
      }
      /// Template parameters are no templates!
      if ((substr($text,$index,3)=="{{{")||(substr($text,$index,3)=="{{#")){
		$offset=$index+3;
		$repeat=true;
      }
    }while ($repeat);
    
    return $index;
  }

  ///-------------------------------------------------------------------


  /**
   * reads the Values from the template usage (e.g. the text between {{ and }})
   * this is needed when starting the editsession and
   * after returning from the default editor.
   *
   * restriction: The given text should only contain the used template
   * not the whole article.
   */
  public static function getValuesFromText( &$templates,$Segments ){
	if (array_key_exists('Templates',$templates) && is_array($templates['Templates'])){
	  foreach(array_keys($templates['Templates']) as $key){
		$text = $Segments[$templates['Templates'][$key]['index']];
		// clsTemplateForm::debug("$key / getting value from: $text");
		self::getTemplateValuesFromText($templates['Templates'][$key],$text);
		
	  }
	}
  }
  
  
  /**
   * parses a single Template  
   * writes the found values into the given "templateFields" array
   * 
   * @param templateFields
   * @param text the textual representation of the templates-useage
   */
  private static function getTemplateValuesFromText(&$templateFields,&$text){   
    $max = count($templateFields);
	
	$values = self::lexer(self::tokenizer($text));

	if (!is_array($values)) {
		echo ("<pre>\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nproblem encountered when parsing template call:\n$values\n".$text."</pre>\n");	
		return;
	}
	//print_r($values);	
	$knownFields = array();
	$fieldNr = array();
    for( $i = 0; $i < $max; $i++ ) {
		if (array_key_exists($i,$templateFields) && is_array($templateFields[$i])){
			$field = $templateFields[$i]['field'];
			if (array_key_exists($field,$values)) {
				$templateFields[$i]['value'] = str_replace("<br>","\n",$values[$field]);
			}
			else {
				$templateFields[$i]['value'] = '';
				$templateFields[$i]['comment'] .= ' <small>missing, will be added</small>';
			}
			$knownFields[$field] = true;
			$fieldNr[$field]=$i;
		}
	}
	// create virtual definition for unknown fields
    foreach ($values as $field => $value) {
		if ($field =='0') continue;
		if (array_key_exists($field,$knownFields)) continue;
		$templateFields[$i] = array();
		$templateFields[$i]['field'] = $field;
		$templateFields[$i]['type']  = '-';
		$templateFields[$i]['comment'] = "<small>undefined field -- WILL BE DELETED</small>";
		$templateFields[$i]['typeParams'] = '()';
		$templateFields[$i]['value'] = str_replace("<br>","\n",$value);
		$fieldNr[$field]=$i;
		$i++;
	}
	self::applyCommands($templateFields,$fieldNr);
  }

  private static function applyCommands(&$templateFields,$fieldNr) {
    for( $i = 0; $i < count($templateFields); $i++ ) {
		if (isset($templateFields[$i]['command'])){
			$command = $templateFields[$i]['command'];
			// "from" will take a parameter´s value from another parameter if it is empty
			if (substr($command,0,4)=='from') {
				$from = preg_replace('/from\s+/','',$command);
				if ($templateFields[$i]['value']=='' && isset($fieldNr[$from])) {
					$templateFields[$i]['value'] = $templateFields[$fieldNr[$from]]['value'];
					$templateFields[$i]['comment'] = "<small>value taken from parameter '$from'</small>";
				}	
			}
		}
	}
	
	
  }
  
  /**
   * breaks the given text into tokens.
   * this tokenizer knows the following tokens:
   * '{', '}', '|', '=', and a* (which is a string of any other character).
   *
   * @param text
   *@returns array of tokens where $text == implode("",$result)
   */
  private static function tokenizer($text){
	$result=array();
	$lastindex=0;
	$brlevel=0; // brackets
	for( $index= 0; $index < strlen($text); $index++){  
	  switch( $text[$index]){
	  case '[': $brlevel++; continue;
	  case ']': $brlevel--; continue;
	  case '{':
	  case '}':		
	  case '=':		
	  case '|':
	    if ($brlevel!=0) continue; // 
		if ($text[$index]=='=' && $text[$lastindex-1]=='=') continue;  // named parms may contain equals sign
		if ($lastindex < $index) {
		  if ($text[$index]=='=') {
			$result[] = substr($text,$lastindex, $index-$lastindex);
		  } else if (clsTemplateForm::$optionUseNewLine) {
			$result[] = substr($text,$lastindex, $index-$lastindex);
		  } else {
			$result[] = substr($text,$lastindex, $index-$lastindex);
		  }
		}
		$lastindex=$index+1;
		$result[]=$text[$index];
		break;
	  default:
		continue;
	  }
	  
	}
	if ($lastindex = $index){
	  $result[] = substr($text,$lastindex, $index-$lastindex);
	}
	
	return $result;
  }

  /**
   * uses the default-wiki-parser for parsing the given text.
   *
   * @returns the result of the default-wiki-parser
   */
  private static function preparse($text){
	global $wgUser,$wgParser;
	$parserOptions = ParserOptions::newFromUser( $wgUser );
	return $wgParser->preprocess
	  ($text,
	   clsTemplateForm::getTemplateForm()->mTitle,
	   $parserOptions);
  }
  
  /**
   * creates the abstract syntax tree of the given template
   */
  private static function lexer($tokens){
	//  BNF: terminals are quoted,  e is the empty word:
	// template ::= "{" "{" value [ "|"  value  ("=" value) ? ]* "}" "}";;
		
	if (( $tokens[0] != "{" )||($tokens[1] !="{")) {
	  clsTemplateForm::debug("Parsing first two tokens fails" );  
	  return "code 1"; //parsing error.
	}
	
	$result = array();
	$r_idx =0;
	$index=2; 
	// print_r($tokens);
	if ($token = self::parseValue($tokens,$index)){
		$token=trim($token);
		$result[$r_idx]=$token;
		$r_idx ++;
	}else {
		return "code 2"; //parsing error
	}
	
	while ($tokens[$index] == "|"){
	  $index++;	
	    
	  if($name = self::parseValue($tokens,$index)) {
		$name=trim($name);
		if (strpos(strip_tags($name),"{{subst:",0) !== false){ 
		  $name =self::preparse($name);
		}
		if ($tokens[$index] == "="){
		  $index++;

		  $value = self::parseValue($tokens,$index);
		  if ( ! ($value === false) ){
			$value=trim($value);
			// print("lexer: template=".$result[0]." name=$name value=$value\n");
			// $result[$name]=self::preparse($value);  -- do not parse because we might loose embedded template calls
			$result[$name]=$value;
		  }else {
			return "code 3";
		  }
		} else{
		  $result[$r_idx]=$name;
		  $r_idx++;
		}
	  } else{
		return "code 4";
	  }
	}
	
	//$index=count($tokens)-2;
	
	if ( ( $tokens[$index] != "}" ) ||
		 ($tokens[$index+1] !="}")) {
	  clsTemplateForm::debug(" Error \n The last two tokens are no '}' !");
	  
	  $result = "code 5 The last two tokens are no '}' -- '".$tokens[$index].",".$tokens[$index+1]."'"; //parsing error.
	}
	return $result;
	
  }

  /**
   * this function merges a number of tokens to one value (which can contain
   * subtemplates or other parserfunctions)
   *
   * @param tokens
   * @param index
   *
   */
  private static function parseValue(&$tokens,&$index){
	// print("entering parseValue:index=$index\n");
	// value ::= "{" ( value | "|" | "=" )* "}" | value* ;;
	// if the Value does not start with "{" 
	// it ends before the first "|" or "}"
	if ($index >= count($tokens))
	  {
	  clsTemplateForm::debug(" Error \n index ".$index." out of range   !");

	  return false; // parsing error;
	  }
	$result = "";
	
	switch($tokens[$index]){
	case "{":
	  
	  $index++;
	  
	  if (!$value= self::parseValue($tokens,$index))
		$result = false;
	  else{
		$result= "{".$value;
		while( ($index < count($tokens))&&
			   ($tokens[$index] !== "}")
			   ) {
		  if(($tokens[$index] == "|")||($tokens[$index] == "=")){
			if ($index >= count($tokens)) {
			  clsTemplateForm::debug(" Error \n index ".$index.
									 " out of range (1) !");
			  return false;
			}
			$result.=$tokens[$index];
			$index++;
		  }elseif($value = self::parseValue($tokens,$index)){
			$result .= $value;
		  }else{
			// unexpected tokens are surrounded by '(?' and '?)'
			$result .= "(?".$tokens[$index]."?)";
			$index++;
		  }
		}
		$index++;
		$result .="}";
	  }

	  break;

	case "|":
	case "=":
	case "}":
	  break;

	default:
		$result.=$tokens[$index];
		$index++;
		for(;;) {
			// print("result=$result, next index=$index, next token=".$tokens[$index]."\n");
			if ($index >= count($tokens)) {
				clsTemplateForm::debug(" Error \n index ".$index.
									 " out of range (2) !");
				return false;
			}
			if ($tokens[$index]=="=" || $tokens[$index]=="}" || $tokens[$index]=="|") break;
			$value = self::parseValue($tokens,$index);
			if (!$value) return false;
			$result .= $value;
		}
	}
	// print("exiting parseValue:index=$index,$result\n");
	return $result;
  }
  /**/

}