<?php
class TemplateForm_UI {

  /**
   * tries to display the form for the given Templates:
   * @param editpage
   * @param templateFields
   */
  function createTemplateFormEditor( &$editpage,&$templateFields,&$pageSegments,&$pageSection) {
	global $wgOut, $wgRequest, $wgParser;
	$clsTypeDefinition = new clsTypeDefinition();
  
	$wgOut->setArticleFlag(false);
	$wgOut->setArticleRelated(true);
	$wgOut->setPageTitle( wfMsg( 'editing',
								 $editpage->mTitle->getPrefixedText()
								 ) );

	$wgOut->AddHtml( $editpage->editFormPageTop); // Before even the preview
	$instance = clsTemplateForm::getTemplateForm();

	self::diffOrPreview($editpage,$templateFields,$instance->mSection);
	$wgOut->AddHtml( $editpage->editFormTextTop);
  
	if ($wgRequest->getText('oldid')>0){
	  wfLoadExtensionMessages( 'TemplateFormEdit' );
	  $editpage->editFormAfterWarn .= '<br>'.wfMsg('oldversion').'<hr>';
	}

	$wgOut->addHTML(self::createHtmlFormTag($editpage,$templateFields,$pageSegments,$pageSection));
  
	return false; 
  }

  /**
   * finds the last headline of the given Text 
   * @param prefix - the text to be inspected
   * @returns parsed headline (ready for using in $wgOut->AddHtml...)
   */
  static function guessHeadline($tplName, $prefix){
	global $wgUser,$wgParser;
	$instance = clsTemplateForm::getTemplateForm();

	$headers=array();
	// headline in the first row.
	$offset = preg_match("/^ *([=]{1,6})[^=]*\\1 *(\\n|$)/si", $prefix,$headers);
	if (count($headers)>0)
	  $header=$headers[0];
	else
	  $header="";	
  
	while(true){
	  $prefix = substr($prefix,$offset+strlen($header));
	  $headers=array();
	  $offset = preg_match("/\\n *([=]{1,6})[^=]*\\1 *(\\n|$)/si", $prefix,$headers);
	  if (count($headers)>0)
		$header=$headers[0];
	  else
		break;
	}
	if(strlen($header)>0){
	
	  $parserOptions = ParserOptions::newFromUser( $wgUser );
	  $prefix = $wgParser->parse("<hr>\n__NOEDITSECTION__".$header,
								 $instance->mTitle,							 
								 $parserOptions)->getText();
	  return $prefix;
	}
	return "<table width=100%><tr><td width=80%><hr></td><td><i>&nbsp;[ $tplName ]</i></td></tr></table>";
  }

  /**
   * insert portions of wiki source into html form 
   */
	private static function insertNonTemplateCode($text,$nr,$min) {
		if (trim($text)=='') return "<input name='wpNonTplText$nr' type='hidden' value='\n' />";
		$rows = substr_count($text,"\n");
		if ($rows>10) $rows=10;
		if ($rows<$min) $rows=$min;
		$html  = "<textarea name='wpNonTplText$nr' cols=80 rows=$rows>";
		$html .= $text;
		$html .= "</textarea>";
		return $html;
	}
	

  /**
   * Creates a String that contains the html <FORM> Tag 
   * it realizes the header and the footer that are independent form the used 
   * templates and calls "createHtmlforTemplate" for each used template in oder
   * to define the template dependent part of the form
   *
   * @param editpage
   * @param templateFields
   * @returns htmlcode of the form
   */
  private function createHtmlFormTag(&$editpage,&$templateFields,&$pageSegments,&$pageSection){

	global $wgRequest;
	wfLoadExtensionMessages( 'TemplateFormEdit' );
    $html ='
<form id="editform" name="editform" method="post" '
      .'action="' . $editpage->mTitle->escapeLocalURL("action=submit") . '" '
      .'enctype="multipart/form-data">
';
	$n="\n";
	$html .= $editpage->editFormTextBeforeContent;

	foreach($templateFields['Templates'] as $inx => $invocation) {
		$index=$invocation['index'];
		$limit = ($index==1) ? 20 : 0;
		$html .= self::insertNonTemplateCode($pageSegments[$index-1],$index-1,$limit);
		if ($index>0) $html.= self::guessHeadline ($invocation['Name'], trim(clsTemplateForm::getTemplateForm()->m_PageSegments[$index-1]));

		$html .= self::createHtmlForTemplate($invocation);
		
	}
	$html .= self::insertNonTemplateCode($pageSegments[$index+1],$index+1,20);
	
	if (!array_key_exists('summary',$templateFields)) $templateFields['summary'] = '';
	$html .= '<table width="100%">'.$n.'  <tr><td colspan=2><hr></td></tr>'.$n.
	  '  <tr>'.$n.'    <td width="1%" warp="nowarp">' . wfMsg('editsummary').
	  '</td><td>'.$n. 
	  self::inputTag('text',"wpSummary", "", $templateFields['summary']) . '</td>'.
	  $n.'  </tr><tr>'.$n.'    <td colspan="2">'.$n.
	  '      <input id="wpMinoredit" name="wpMinoredit" type="checkbox"  ';
	if ($wgRequest->getCheck('wpMinoredit')) $html .='checked="checked"';
	
	$html .='/>'. wfMsg('minoredit').$n.
	  '      <input id="wpWatchthis" name="wpWatchthis" type="checkbox"  ';
	
	if ($wgRequest->getCheck('wpWatchthis')) $html .='checked="checked"';
	
	$html .='/>'. wfMsg('watchthis').$n."    </td>\n  </tr>\n";
	$html .="</table>\n" . $editpage->editFormTextAfterWarn;
	$html .= self::inputTag('submit', "wpSave", "s", wfMsg('savearticle'))
	  . self::inputTag('submit',"wpPreview" , "p", wfMsg('showpreview'))
	  . self::inputTag('submit',"wpDiff"    , "v", wfMsg('showdiff'))
	  . self::inputTag('submit',"wpDisable" , "", wfMsg('wikitext'))
	  . self::inputTag('hidden',"section", "", $pageSection)
	  . self::inputTag('hidden',"wpTextbox1", "", base64_encode($editpage->textbox1))
	  . self::inputTag("hidden","TemplateEditBox","", "used")
	  .' <a href="'. $editpage->mTitle->getFullUrl("").'">'
	  . wfMsg('cancel').'</a>';
	
	$html .= $editpage->editFormTextAfterTools . $editpage->editFormTextBottom;
  	$html .= '</form>';  
    return $html;
  }

  /**
   * @returns the html-code for a default inputfield
   */
  private static function inputTag($type,$name,$accesskey,$text) {
	if ($accesskey=='') return '   <input id="'.$name.'" name="'.$name.'"'.' type="'.$type.'" value="'. $text . '" />'."\n";
	return '   <input id="'.$name.'" name="'.$name.'" accesskey="'.$accesskey.'" type="'.$type.'" value="'. $text . '" />'."\n";
  }
  
  /**
   * generates only the fields for one template and nothing else at all. 
   *
   * this returns only the part of the Form for one Template
   * it does not contain the <form> tag and not the submit-buttons 
   * and no hidden elements of the form.
   *
   * @param templateFields
   * @returns html code
   */

  private static function createHtmlForTemplate($invocation) {
    global $wgParser;

	static $instance;
	if (! $instance) $instance = new clsTypeDefinition();

	$templateName =$invocation['Name'].$invocation['index'].'_';
	
	$html ="    <!-- Template ".$templateName." -->\n";


	// handle fields / parameters of invocation
	
	foreach ($invocation as $i => $invoc) {
		if (is_array($invoc)) {

			// create default assumptions for undefined parameters
			if (!array_key_exists('type',$invoc)) 	$invoc['type'] = '';
			if (!array_key_exists('type',$invoc)) 	$invoc['type'] = '';
			if (!array_key_exists('label',$invoc)) 	$invoc['label'] = '';
			if (!array_key_exists('comment',$invoc))$invoc['comment'] = '';
			if (!array_key_exists('typeParams',$invoc)) $invoc['typeParams'] = '()';
			$label = $invoc['field'];
			$usedDefinition[$label] = $label;
			if (strlen($invoc['label']) > 0 )  $label =$invoc['label'];
			$html .= 
			  "<tr><td bgcolor=#ddffdd align=right>&nbsp;". $label ." &nbsp;</td>\n".
			  "<td>".$instance->createHtmlInputElement($invoc,$templateName)."</td>\n".
			  "<td>".$invoc['comment']."</td></tr>\n";			  
		}
	}
	
	return "\n ".' <table>'."\n".$html."\n  </table>";
  }

  ///----------------------------------------------------------------------
  
  
  /**
   * if the user pressed Diff or Preview this method shows it.
   *
   * @param editpage
   * @param templateFields
   * @param Section
   */
  private static function diffOrPreview(&$editpage,$templateFields,$Section){
	global $wgRequest,$wgOut,$wgUser,$wgParser;
	
	$editpage->textbox1= 
	  clsTemplateForm::getTemplateForm()->createArticle($templateFields);
	
	if( $wgRequest->getCheck( 'wpDiff' ) ) {
	  $wgOut->AddHTML( $editpage->showDiff() );
	} else if( $wgRequest->wasPosted() && $wgRequest->getCheck('wpPreview')){	
	  $parserOptions = ParserOptions::newFromUser( $wgUser );
	  $parserOptions->setEditSection( $Section ); //todo;
	  $parserOptions->setTidy(true);
	  $output ='<div style="border: 2px solid #e7e777; '
		. 'padding: 8px; margin: -3px;">';
	  $output.= $wgParser->parse( $editpage->textbox1, 
								  $editpage->mTitle, 
								  $parserOptions )
		->getText();
	  
	  $wgOut->AddHtml($output . '</div><br>');
	}
  }
  /**/
}