<?php
/**
 * this class contains functions for the Template HTML-FORM Editor
 * Extension
 *
 */

require_once(dirname(__FILE__)."/TemplateForm.UI.php");
require_once(dirname(__FILE__)."/TemplateForm.Parser.php");

class clsTemplateForm {

  // static wrapper for "createEditor"
  static function eventhandler( &$editpage ) {
	
	if ( ! self::checkUserRights($editpage) )  return false;
	global $wgRequest;
	
	return self::getTemplateForm()->createEditor($editpage);
  }


  //// ///////////// begin region " member variables " /////////////////// ////

  /**
   * 'false':   the template is written into a single line   
   * 'true': every parameter is written into its own line.
   */
  var $optionUseNewLine=false;

  /**
   * stores the whole article in order to write untouched parts of the article back after clicking "save".
   */
  var $m_PageSegments;

  /**
   * the Title of the page which is edited.
   */
  var $mTitle;// type: Title;

  /**
   * the ID of the Section to be edited (still does not work correctly)
   */
  var $mSection;

  //// ////////////// end region " member variables " //////////////////// ////


  //// ///////////// begin region "singleton pattern" //////////////////// ////
  /*PRIVATE constructor ?*/

  /**
   * Singleton Instance Getter
   * @return the only instance of this class
   */
  function getTemplateForm(){
	static  $iTemplateForm;
    if (!$iTemplateForm)
      $iTemplateForm = new clsTemplateForm;
    return  $iTemplateForm;
  }
  //// ////////////// end region "singleton pattern" ///////////////////// ////


  /**
   *  Initialization of the instance of clsTemplateForm 
   */  
  function initialization(&$editpage){
	global $wgRequest;
	$this->mTitle = $editpage->mTitle;  
	if($wgRequest->getText('section')){
	  $this->mSection = $wgRequest->getText('section');
	}else{
	  $this->mSection = false;
	}
  }



  //// ///////// "methods not affecting the template form directly" ////// ////

  /**
   * Checks whether the current user is allowed to edit (or create) the $editpage.
   * if the user does not have the right the corresponding message is shown.
   * if the user has all needed rights the function returns 'true'
   * @param &$editpage  the page the user wants to change,
   * @return boolean  - false exactly in that case an error message is shown.
   */
  static function checkUserRights(&$editpage) {
    global $wgOut, $wgUser;
   
    if ( !$editpage->mTitle->userCanEdit() ) {
      $wgOut->readOnlyPage( $editpage->mArticle->getContent(), true );
      return false;
    }
   
    if ( !$wgUser->isAllowed('edit') ) {
      if ( $wgUser->isAnon() ) {
		$editpage->userNotLoggedInPage();
		return false;
      } else {
		$wgOut->readOnlyPage( $editpage->mArticle->getContent(), true );
		return false;
      }
    }
    if ( !$editpage->mTitle->userCan( 'create' ) && 
		 !$editpage->mTitle->exists() ) {
      $editpage->noCreatePermission();
      return false;
    }
    return true;
  }

  /**
   * this method assumes that the given $text starts with "{{" and 
   * ends with "}}". It ignores prefix "subst:" if it exists.
   *
   * @param text of a template-usage
   * @return name of given Template
   */
  function getTemplateName($text){
      
    $end = strpos($text,"|");
    if ((!$end) || ( strpos($text,"}}")< $end)) $end=strpos($text,"}}");
    
    // care for line breaks
    if ((!$end) || ( strpos($text,"\n")&&
					 (strpos($text,"\n")< $end)))
	  $end=strpos($text,"\n");
    if (substr($text,2,6) == "subst:")
      $start = 8;
    else
      $start = 2;
    return substr($text,$start,$end-$start);
  }

  /**
   * checks whether the button "<WikiText>" was pushed.
   * if so, the Request is prepared for calling the
   * default editor.
   * @param $allTemplates  - is needed to write the latest changes into the
   *                           manipulated request.
   *
   * @return true, if the FormEditor should not be used.
   * @return false, in any other case 
   */
  function isChecked_WikiText_Button($allTemplates){
    global $wgRequest;
	
    if(  ($wgRequest->getText("TemplateEditBox") == "used") &&
		 ( $wgRequest->getCheck( 'wpDisable' ) ) )
      {
		self::getValuesFromRequest($allTemplates);
		$data = $wgRequest->getValues();
		$data['wpTextbox1'] = $this->createArticle($allTemplates);
		$wgRequest = new FauxRequest($data,true);
		return true;    
      }
    return false;
  }

  //// ///////// end region methods that do not affect the template form directly ///////// ////
  
  /**
   * Retrieves text from an article
   * @param name of the article as string
   * @return text from article as string
   */
  function getArticleText( $article = "Main Page" ,$oldid = 0,$section = false) {
	global $wgParser;

	$text='';

	$t = Title::NewFromText($article);
	$art = Revision::newFromTitle( $t ,$oldid);
	if( $art != null ) {
	  $text = $art->getText();
	} 
    if ($section){
	  $text = $wgParser->getSection($text,$section,"");
    }
	return $text;
 }
  
  /**
   * generates the text to be written into the article when you click "save"  or "wikitext"
   */
  function createArticle($templates){
	
	foreach($templates['Templates'] as $templateFields){
	  $index=$templateFields['index'];
	  
	  $template = "{{" . $templateFields['Name'];
	  
	  if ($this->optionUseNewLine)
		$template .= "\n";
	  for( $i = 0; $i < count($templateFields); $i++ ) 
		if (is_array($templateFields[$i])){
		  $template.= '|';
		  $template.= $templateFields[$i]['field'];
		  $template.= '=';
		  $template.= str_replace("\n","<br>",$templateFields[$i]['value']);
		  if ($this->optionUseNewLine)
			$template .= "\n";
		}
	  $template .= "}}";
	  $this->m_PageSegments[$index] =$template;
	  	  
	}

	$templates = $this->m_PageSegments;
	  
	return implode("",$templates);
  }

  //// //////// begin region "getValues" //////////////// 
 
  /** 
   * reads the latest changes entered before
   * 'preview' or 'safe' was clicked from the Request sended by
   * submitting the form...
   ******************************************* REQUEST
   */  
  static function getValuesFromRequest(&$templates){
	global $wgRequest;
	foreach(array_keys($templates['Templates']) as $key){
	  self::getTemplateValuesFromRequest($templates['Templates'][$key]);
	}
    $templates['summary'] = $wgRequest->getText( 'wpSummary' );  

  }


  /**
   * gets the Values of one Template from the current request.
   * @param tamplateFields - the given array is changed by this function.
   * no return value
   ******************************************* REQUEST
   */
  private static function getTemplateValuesFromRequest(&$templateFields){
    global $wgRequest;

	$fieldPrefix = 'wp' . $templateFields['Name'].$templateFields['index']."_";
	for( $i = 0; $i < count($templateFields); $i++ ) 
      if (is_array($templateFields[$i])) {
		$plainFieldName = $templateFields[$i]['field'];
		$formFieldName =  $fieldPrefix. $plainFieldName;
		
		if ($templateFields[$i]['type'] == 'someof'){
		  $typeParams=$templateFields[$i]['typeParams'];
		  $count=$wgRequest->getText($formFieldName);

		  $Value = "";	  
		  for ($id=0 ; $id < $count ; $id++)
			if($idValue = $wgRequest->getText($formFieldName.'_'.$id)){
			  if ($Value !== "")
				$Value.= ', ';
			  $Value.= $idValue;
			}
		  $templateFields[$i]['value'] = $Value;  
		} else 
		  $templateFields[$i]['value'] = $wgRequest->getText( $formFieldName ); 
      }
	  
  }
  
  
  
  
  /**
   * gets the values of the templates 
   *from the request or from the the article
   ******************************************* REQUEST
   */
  public function getTemplateValues(&$templates){
	global $wgRequest;
	
	if ( ( $wgRequest->wasPosted() ) && 
		 ($wgRequest->getText("TemplateEditBox") == "used") 
		 ){
	  self::getValuesFromRequest($templates);

	}else{ 
	  TemplateFormParser::getValuesFromText($templates,$this->m_PageSegments);
	}
  }
  //// ////////// end region "get Values" ////////////// ////

  /**
   * Writes Debug information directly into the output-html-file. 
   */  
  public static function debug($text){
	//global $wgOut;
	//$prepared = str_replace("<","&lt;",str_replace(">","&gt;",$text));
	//$wgOut->addHtml("<pre>".$prepared."</pre>");
  }


  /**
   * saves the whole article with all latest changes.
   * @param editpage
   * @param templateFields
   */ 
  function save(&$editpage,&$templateFields){
	global $wgParser,$wgRequest,$wgOut;
		
	self::getValuesFromRequest($templateFields);
	$data=$this->createArticle($templateFields);

	$summary=$templateFields['summary'];

	if( $editpage->mTitle->exists() )  $flags = EDIT_UPDATE;
	else $flags = EDIT_NEW;
  
	$dbw = wfGetDB( DB_MASTER );
	
	if ($wgRequest->getCheck('wpWatchthis')) {
	  if (!$editpage->mArticle->mTitle->userIsWatching()) {
		$dbw->begin();
		$editpage->mArticle->doWatch();
		$dbw->commit();
	  }
	} else {
	  if ( $editpage->mArticle->mTitle->userIsWatching() ) {
		$dbw->begin();
		$editpage->mArticle->doUnwatch();
		$dbw->commit();
	  }
	}
	if ($this->mSection){
	  $data = $editpage->mArticle->replaceSection($this->mSection, $data, $summary );
	}   
	if ( $wgRequest->getCheck('wpMinoredit') )
	  $flags |= EDIT_MINOR;
	
	$editpage->mArticle->doEdit( $data, $summary, $flags, false );
	$wgOut->redirect($editpage->mArticle->mTitle->getFullURL( ));
  }

  //---------event handling of the event "Alternate Edit" --------


  /**    
   * if the request was invoked by the TemplateFormEditor the 
   * latest changes are in the several input fields and the
   * parts not belonging to the edited template-usage are
   * base64 encoded in the hidden field wpTextbox1
   *
   * if the request was invoked by an other editor 
   * the latest changes and the context of the page
   * can be found in wpTextbox1 without base64 encoding
   */
  function createEditor(&$editpage){
	global $wgRequest , $wgParser, $wgOut;
	
	$resultBoolean = true;    
	$text="";
	$this->initialization($editpage);
	

	if ( $wgRequest->wasPosted() ) {      
	  
	  // "Save","Preview", "View Changes" or "<WikiText>" was clicked.
	  $text = $wgRequest->getText( 'wpTextbox1' );
	  if ($wgRequest->getText("TemplateEditBox") == "used"){ 
		$text = base64_decode($text);
	  }else{
		if( $wgRequest->getCheck( 'wpSave' ) ) 
		  return true;
	  }	
	} else {
	  // the user has clicked "edit" on an existing article 
	  
	  $titlestring =$editpage->mTitle->getFullText();
	  $oldid="";
	  $preload=false;
	  
	  if(!$editpage->mTitle->exists()){
		$preload = $wgRequest->getText('preload');
		$value=1; //new page
	  }else{
		$oldid = $wgRequest->getText('oldid');
		$value=2; //oldversion
	  }
	  
	  if ($preload){
	  
		// this handles preload exactly as Template:CreateArticle does it.
		$parserOptions = ParserOptions::newFromUser( $wgUser );
      
		$text = trim($this->getArticleText($preload));
	  
		$text =preg_replace('~</?(includeonly|onlyinclude|nowiki)>~','',$text);
	  
	  }else {
	  
		$text = trim($this->getArticleText($titlestring,$oldid,$this->mSection));
  
	  }
  
	}
	$this->m_PageSegments =	TemplateFormParser::splitByTemplate($text);
 
	$allTemplates = array();
	$allTemplates['Templates']= array();
	
	for ( $index = 0 ; $index < count( $this->m_PageSegments ) ; $index++ ){
	  if ($fields = self::isFormTemplate($this->m_PageSegments[$index]))  {
		$fields['index']=$index;
		$allTemplates['Templates'][]=$fields;
		//break;
	  }
	}
	if ( $this->isChecked_WikiText_Button($allTemplates) )	return true;
  
	if( $wgRequest->getCheck( 'wpSave' ) ){
 
	  $this->save($editpage,$allTemplates);
	
	  return false;
	}
	if (count($allTemplates['Templates']) > 0){
	  self::getTemplateValues($allTemplates);

	  TemplateForm_UI::createTemplateFormEditor($editpage,$allTemplates);
	  return false;
	} 
	else
	  return true;

  }

  /**
   * detemines if a form can be created for the given template
   * returns false if not
   * returns an array if it is possible
   *  the array contains some information about the template.
   */
  static function isFormTemplate($template){
	if (!(substr($template,0,2) === "{{"))
	  return false;  

	$templateName = self::getTemplateName($template);  

	if(!$templateTitle = Title::NewFromText("Template:".$templateName))
	  return false;
	if(!$templateTitle->exists())
	  return false;

	$clsTypeDefinition = new clsTypeDefinition();
  
	if ($templateFields = $clsTypeDefinition->getTemplateFields($templateName)) 
	  return $templateFields;

	return false;
  }
  /**/ 
}
?>