<?php
/**
 * this class contains the parser for Parsing...
 * - the Request's wpTextbox1's value when returning from the standard editor
 * - the editpage when edit session is started.
 * - the values of the currently inspected template
 *
 * - NOT the Template Definition Page which is already inspected by
 *       the class clsTypeDefinition
 */
class TemplateFormParser{

  /* INTERFACE:
   * static :   splitByTemplate ( $ text )
   * static :   getValuesFromText ( & $ templates , $ Segments )
   *
   */

  /**
   * finds all template-usages in $text the result is an array
   * of segments of $text 
   * assured is:  $text == implode("",splitByTemplate($text));
   *
   * @param $text - any wikitext
   *
   * @return an array containing the same text as the input parameter
   *       - broken into several sections
   */
  public static function splitByTemplate($text){
    $result = null;
    
    $index = self::findnexttemplate($text, 0);
    
    // zeichen vor dem ersten Template
    $result[] = substr($text,0,$index);
    $text=substr($text,$index);
    
    while( (strlen($text)>0) && (strpos($text,"{{") == 0) ){
	  
      $i = 2;
      $brackets =2; // die zwei öffnenden Klammern des Templates.
      $max =strlen($text);
      
      for(;($i < $max) && ($brackets > 0);$i++){
		$char = substr($text,$i,1);
		if ($char=="{") $brackets++;
		if ($char=="}") $brackets--;
      }
      
	  $template =substr($text,0,$i);
	  if(clsTemplateForm::isFormTemplate($template)){
		$result[]=substr($text,0,$i);
		$text = substr($text, $i);
		$i=0;
      }
	  
      //----------------------------------------------------------//
      $index = self::findnexttemplate($text, $i);
      
	  if ( $i > 0)
		$result[count($result)-1] .= substr($text,0,$index);
	  else if ( $index > 0)
		$result[] = substr($text,0,$index);
      
      $text=substr($text,$index);
    }

	//Debug
	foreach ( $result as $item)
	  clsTemplateForm::debug("p) ".$item);
	return $result;
  }



  /**
   * finds the first beginning of a template which is not enclosed by <nowiki>
   *
   * @param text
   * @param offset
   *
   * returns the position of the next Template inside $text beginning 
   *         the search after $offset
   */
  private static function findnexttemplate(&$text,$offset){
	
    do{
      $repeat = false;
      $index = strpos($text,"{{",$offset);
      if ( ($index) || (substr($text, $offset, 2) == "{{") ) {
		
		$tag = strpos($text,"<nowiki>",$offset);
		if ( ($tag) || (substr($text,$offset,8) == "<nowiki>" )){
		  if ($tag < $index){      
			// möglicherweise ist das nächste Template 
			//in <nowiki> eingeschlossen.
			if( strpos($text,"</nowiki>",$tag) ){
			  $depth=0;
			  $tag=$tag+2;
			  
			  do{
				if ((strpos($text,"<nowiki>",$tag))
					&& ( strpos($text,"<nowiki>",$tag)
						 < strpos($text,"</nowiki>",$tag))){
				  $tag = strpos($text,"<nowiki>",$tag)+2;
				  $depth++;
				}
				if (strpos($text,"</nowiki>",$tag))
				  if (( strpos($text,"</nowiki>",$tag)
						< strpos($text,"<nowiki>",$tag))
					  ||(!strpos($text,"<nowiki>",$tag))){
					$tag = strpos($text,"</nowiki>",$tag);
					$depth--;
				  }
			  }while($depth > 0);
			  
			  //	      if ($depth==0){
			  $tag = strpos($text,"</nowiki>",$tag);
			  //	      }
			  $offset=$tag;
			  $repeat = true;
			}
			else 
			  return $index;
		  }
		}
      } else {
		$index=strlen($text);
      }
      /// Template parameters are no templates!
      if ((substr($text,$index,3)=="{{{")||(substr($text,$index,3)=="{{#")){
		$offset=$index+3;
		$repeat=true;
      }
    }while ($repeat);
    
    return $index;
  }

  ///-------------------------------------------------------------------


  /**
   * reads the Values from the template usage (e.g. the text between {{ and }})
   * this is needed when starting the editsession and
   * after returning from the default editor.
   *
   * restriction: The given text should only contain the used template
   * not the whole article.
   */
  public static function getValuesFromText( &$templates,$Segments ){
	if (is_array($templates['Templates'])){
	  foreach(array_keys($templates['Templates']) as $key){
		$text = $Segments[$templates['Templates'][$key]['index']];
		self::getTemplateValuesFromText($templates['Templates'][$key],$text);
		
	  }
	}
  }
  
  
  /**
   * parses a single Template  
   * writes the found values into the given "templateFiels" array
   * 
   * @param templateFields
   * @param text the textual representation of the templates-useage
   */
  private static function getTemplateValuesFromText(&$templateFields,&$text){   
    $max = count($templateFields);
	
	$values = self::lexer(self::tokenizer($text));
	
    for( $i = 0; $i < $max; $i++ ) 
      if (is_array($templateFields[$i])){
		$templateFields[$i]['value'] = 
		  str_replace("<br>","\n",$values[$templateFields[$i]['field']]);	
	  }
  }

  /**
   * breaks the given text into tokens.
   * this tokenizer knows the following tokens:
   * '{', '}', '|', '=', and a* (which is a string of any other character).
   *
   * @param text
   *@returns array of tokens where $text == implode("",$result)
   */
  private static function tokenizer($text){
	$result=array();
	$lastindex=0;
	for( $index= 0; $index < strlen($text); $index++){  
	  switch( $text[$index]){
	  case '{':
	  case '}':		
	  case '=':		
	  case '|':
		if ($lastindex < $index){
		  $result[] = substr($text,$lastindex, $index-$lastindex);
		  }
		$lastindex=$index+1;
		$result[]=$text[$index];
		break;
	  default:
		continue;
	  }
	  
	}
	if ($lastindex < $index){
	  $result[] = substr($text,$lastindex, $index-$lastindex);
	}
	
	return $result;
  }

  /**
   * uses the default-wiki-parser for parsing the given text.
   *
   * @returns the result of the default-wiki-parser
   */
  private static function preparse($text){
	global $wgUser,$wgParser;
	$parserOptions = ParserOptions::newFromUser( $wgUser );
	return $wgParser->preprocess
	  ($text,
	   clsTemplateForm::getTemplateForm()->mTitle,
	   $parserOptions);
  }
  
  /**
   * creates the abstract syntax tree of the given template
   */
  private static function lexer($tokens){
	//  BNF: terminals are quoted,  e is the empty word:
	// template ::= "{" "{" value [ "|"  value  ("=" value) ? ]* "}" "}";;
		
	if (( $tokens[0] != "{" )||($tokens[1] !="{")) {
	  clsTemplateForm::debug("Parsing first two tokens fails" );  
	  return false; //parsing error.
	}
	
	$result = array();
	$r_idx =0;
	$index=2; 
	
	if ($token = self::parseValue($tokens,$index)){
	  $result[$r_idx]=$token;
	  $r_idx ++;
	}else
	  return false; //parsing error;
	
	while ($tokens[$index] == "|"){
	  $index++;	
	    
	  if($name = self::parseValue($tokens,$index)){

		if (strpos(strip_tags($name),"{{subst:",0) !== false){ 
		  $name =self::preparse($name);
		}
		if ($tokens[$index] == "="){
		  $index++;
		  $value = self::parseValue($tokens,$index);
		  if ( ! ($value === false) ){
			$result[$name]=self::preparse($value);
		  }else {
			return false;
		  }
		} else{
		  $result[$r_idx]=$name;
		  $r_idx++;
		}
	  } else{
		return false;
	  }
	}
	
	//$index=count($tokens)-2;
	
	if ( ( $tokens[$index] != "}" ) ||
		 ($tokens[$index+1] !="}")) {
	  clsTemplateForm::debug(" Error \n The last two tokens are no '}' !");
	  
	  $result = false; //parsing error.
	}
	return $result;
	
  }

  /**
   * this function merges a number of tokens to one value (which can contain
   * subtemplates or other parserfunctions)
   *
   * @param tokens
   * @param index
   *
   */
  private static function parseValue(&$tokens,&$index){
	// value ::= "{" ( value | "|" | "=" )* "}" | value* ;;
	// if the Value does not start with "{" 
	// it ends before the first "|" or "}"
	if ($index >= count($tokens))
	  {
	  clsTemplateForm::debug(" Error \n index ".$index." out of range   !");

	  return false; // parsing error;
	  }
	$result = "";
	
	switch($tokens[$index]){
	case "{":
	  
	  $index++;
	  
	  if (!$value= self::parseValue($tokens,$index))
		$result = false;
	  else{
		$result= "{".$value;
		while( ($index < count($tokens))&&
			   ($tokens[$index] !== "}")
			   ) {
		  if(($tokens[$index] == "|")||($tokens[$index] == "=")){
			if ($index >= count($tokens)) {
			  clsTemplateForm::debug(" Error \n index ".$index.
									 " out of range  !");
			  return false;
			}
			$result.=$tokens[$index];
			$index++;
		  }elseif($value = self::parseValue($tokens,$index)){
			$result .= $value;
		  }else{
			// unexpected tokens are surrounded by '(?' and '?)'
			$result .= "(?".$tokens[$index]."?)";
			$index++;
		  }
		}
		$index++;
		$result .="}";
	  }

	  break;

	case "|":
	case "=":
	case "}":
	  break;

	default:
	  $result=$tokens[$index];
	  $index++;
	  //$result .= self::parseValue($tokens,$index);
	  break;
	}
	//	clsTemplateForm::debug("value: [vor ".$index.'] '.$result);
	return $result;
  }
  /**/

}