<?php
class TemplateForm_UI {




  /**
   * tries to display the form for the given Templates:
   * @param editpage
   * @param templateFields
   */
  function createTemplateFormEditor( &$editpage,&$templateFields ) {
	global $wgOut, $wgRequest;
	$clsTypeDefinition = new clsTypeDefinition();
  
	$wgOut->setArticleFlag(false);
	$wgOut->setArticleRelated(true);
	$wgOut->setPageTitle( wfMsg( 'editing',
								 $editpage->mTitle->getPrefixedText()
								 ) );

	$wgOut->AddHtml( $editpage->editFormPageTop); // Before even the preview
	
	self::diffOrPreview($editpage,$templateFields,$instance->mSection);
	$wgOut->AddHtml( $editpage->editFormTextTop);
  
	if ($wgRequest->getText('oldid')>0){
	  wfLoadExtensionMessages( 'TemplateFormEdit' );
	  $editpage->editFormAfterWarn .= '<br>'.wfMsg('oldversion').'<hr>';
	} 

	$wgOut->addHTML(self::createHtmlFormTag($editpage,$templateFields));
  
	return false; 
  }

  /**
   * finds the last headline of the given Text 
   * @param prefix - the text to be inspected
   * @returns parsed headline (ready for using in $wgOut->AddHtml...)
   */
  static function guessHeadline($prefix){
	global $wgUser,$wgParser;
	$instance = clsTemplateForm::getTemplateForm();

	$headers=array();
	// headline in the first row.
	$offset = preg_match("/^ *([=]{1,6})[^=]*\\1 *(\\n|$)/si", $prefix,$headers);
	if (count($headers)>0)
	  $header=$headers[0];
	else
	  $header="";	
  
	while(true){
	  $prefix = substr($prefix,$offset+strlen($header));
	  $headers=array();
	  $offset = preg_match("/\\n *([=]{1,6})[^=]*\\1 *(\\n|$)/si", $prefix,$headers);
	  if (count($headers)>0)
		$header=$headers[0];
	  else
		break;
	}
	if(strlen($header)>0){
	
	  $parserOptions = ParserOptions::newFromUser( $wgUser );
	  $prefix = $wgParser->parse("<hr>\n__NOEDITSECTION__".$header,
								 $instance->mTitle,							 
								 $parserOptions)->getText();
	  return $prefix;
	}
	return "<hr>";
  }


  /**
   * Creates a String that contains the html <FORM> Tag 
   * it realizes the header and the footer that are independend form the used 
   * templates and calls "createHtmlforTemplate" for each used template in oder
   * to define the template dependent part of the form
   *
   * @param editpage
   * @param templateFields
   * @returns htmlcode of the form
   */
  private function createHtmlFormTag(&$editpage,&$templateFields){
	global $wgRequest;
	wfLoadExtensionMessages( 'TemplateFormEdit' );
    $html .='
<form id="editform" name="editform" method="post" '
      .'action="' . $editpage->mTitle->escapeLocalURL("action=submit") . '" '
      .'enctype="multipart/form-data">
';
	$n="\n";
	$html .= $editpage->editFormTextBeforeContent;
	foreach($templateFields['Templates'] as $template){

	  $index=$template['index'];
	  if ($index>0)
		$html.= self::guessHeadline
		  (trim(clsTemplateForm::getTemplateForm()
				->m_PageSegments[$index-1]));
	  

	  $html .= self::createHtmlForTemplate($template);	  
	}	
	$html .= '<table width="100%">'.$n.'  <tr><td colspan=2><hr></td></tr>'.$n.
	  '  <tr>'.$n.'    <td width="1%" warp="nowarp">' . wfMsg('editsummary').
	  '</td><td>'.$n. 
	  self::inputTag('text',"wpSummary", $templateFields['summary']) . '</td>'.
	  $n.'  </tr><tr>'.$n.'    <td colspan="2">'.$n.
	  '      <input id="wpMinoredit" name="wpMinoredit" type="checkbox"  ';
	if ($wgRequest->getCheck('wpMinoredit')) $html .='checked="checked"';
	
	$html .='/>'. wfMsg('minoredit').$n.
	  '      <input id="wpWatchthis" name="wpWatchthis" type="checkbox"  ';
	
	if ($wgRequest->getCheck('wpWatchthis')) $html .='checked="checked"';
	
	$html .='/>'. wfMsg('watchthis').$n."    </td>\n  </tr>\n";
	$html .="</table>\n" . $editpage->editFormTextAfterWarn;
	$html .= self::inputTag('submit', "wpSave",wfMsg('savearticle'))
	  . self::inputTag('submit',"wpPreview" ,wfMsg('showpreview'))
	  . self::inputTag('submit',"wpDiff"    ,wfMsg('showdiff'))
	  . self::inputTag('submit',"wpDisable" ,wfMsg('wikitext'))
	  . self::inputTag('hidden',"section", $this->mSection)
	  . self::inputTag('hidden',"wpTextbox1", 
								 base64_encode($editpage->textbox1))
	  . self::inputTag("hidden","TemplateEditBox","used")
	  .' <a href="'. $editpage->mTitle->getFullUrl("").'">'
	  . wfMsg('cancel').'</a>';
	
	$html .= $editpage->editFormTextAfterTools . $editpage->editFormTextBottom;
  	$html .= '</form>';  
    return $html;
  }

  /**
   * @returns the html-code for a default inputfield
   */
  private static function inputTag($type,$name,$text){
	return '   <input id="'.$name.'" name="'.$name.'"'.
	  ' type="'.$type.'" value="'. $text . '" />'."\n";
  }
  
  /**
   * generates only the fields for one template and nothing other at all. 
   *
   * this returns only the part of the Form for one Template
   * it does not contain the <form> tag and not the submit-buttons 
   * and no hidden elements of the form.
   *
   * @param templateFields
   * @returns html code
   */

  private static function createHtmlForTemplate(&$templateFields){
	static $instance;
	if (! $instance) 
	  $instance = new clsTypeDefinition();
	
	$templateUsed =$templateFields['Name'].$templateFields['index'].'_';
	
	$html ="    <!-- Template ".$templateUsed." -->\n";
			
	for( $i = 0; $i < count($templateFields); $i++ ) 
	  if (is_array($templateFields[$i])) {
		$label = $templateFields[$i]['field'];
    	if ( strlen($templateFields[$i]['label']) > 0 )
		  $label =$templateFields[$i]['label'];
		$html .= 
		  '    <tr><td>'. $label ."</td>\n".
		  "      <td>"
		  .$instance->createHtmlInputElement($templateFields[$i],$templateUsed)
		  ."\n      ".$templateFields[$i]['comment']."\n".
		  "      </td>\n".
		  "    </tr>\n";
	  }
	return "\n ".' <table>'."\n".$html."\n  </table>";
  }

  ///----------------------------------------------------------------------
  
  
  /**
   * if the user pressed Diff or Preview this method shows it.
   *
   * @param editpage
   * @param templateFields
   * @param Section
   */
  private static function diffOrPreview(&$editpage,$templateFields,$Section){
	global $wgRequest,$wgOut,$wgUser,$wgParser;
	
	$editpage->textbox1= 
	  clsTemplateForm::getTemplateForm()->createArticle($templateFields);
	
	if( $wgRequest->getCheck( 'wpDiff' ) ) {
	  $wgOut->AddHTML( $editpage->showDiff() );
	} else if( $wgRequest->wasPosted() && $wgRequest->getCheck('wpPreview')){	
	  $parserOptions = ParserOptions::newFromUser( $wgUser );
	  $parserOptions->setEditSection( $Section ); //todo;
	  $parserOptions->setTidy(true);
	  $output ='<div style="border: 2px solid #e7e777; '
		. 'padding: 8px; margin: -3px;">';
	  $output.= $wgParser->parse( $editpage->textbox1, 
								  $editpage->mTitle, 
								  $parserOptions )
		->getText();
	  
	  $wgOut->AddHtml($output . '</div><br>');
	}
  }
  /**/
}